/*
 * Mach Operating System
 * Copyright (c) 1992 Carnegie Mellon University
 * All Rights Reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie Mellon
 * the rights to redistribute these changes.
 */
/*
 * Generic interrupt priority definitions for m68k processors
 *
 * Author: Bryan Ford
 */

#ifndef _m68k_machparam_
#define _m68k_machparam_


/* Set the interrupt priority level and save the old level */
static inline int spln(unsigned short spl)
{
  volatile unsigned short sr;

  asm("movew sr,%0" : "=d" (sr));
  asm("movew %0,sr" : : "d" (spl | (sr & 0xf8ff)));
  return(sr & 0x700);
}


/* Set the interrupt priority level back to the previous level */
static inline void splx(unsigned short pri)
{
  spln(pri);
}


/* Generic spl definitions for m68k interrupt priorities */
static inline int spl7(void)
{
  volatile unsigned short sr;

  asm("movew sr,%0; oriw #0x0700,sr" : "=d" (sr));
  return(sr & 0x700);
}

#define spl6() spln(0x0600)
#define spl5() spln(0x0500)
#define spl4() spln(0x0400)
#define spl3() spln(0x0300)
#define spl2() spln(0x0200)
#define spl1() spln(0x0100)

static inline int spl0(void)
{
  unsigned short sr;

  asm("movew sr,%0; andiw #0xf8ff,sr" : "=d" (sr));
  return(sr & 0x700);
}


#endif _m68k_machparam_
